Package: bibliometrix
Type: Package
Title: Comprehensive Science Mapping Analysis
Version: 5.3.0
Authors@R: c(
    person(given = "Massimo",
           family = "Aria",
           role = c("cre", "aut", "cph"),
           email = "aria@unina.it",
           comment = c(ORCID = "0000-0002-8517-9411")),
    person(given = "Corrado", 
           family = "Cuccurullo", 
           role = "aut",
           email = "cuccurullocorrado@gmail.com",
           comment = c(ORCID = "0000-0002-7401-8575")))
Description: Tool for quantitative research in scientometrics and bibliometrics.
    It implements the comprehensive workflow for science mapping analysis proposed in Aria M. and 
    Cuccurullo C. (2017) <doi:10.1016/j.joi.2017.08.007>.
    'bibliometrix' provides various routines for importing bibliographic data from 'SCOPUS',
    'Clarivate Analytics Web of Science' (<https://www.webofknowledge.com/>), 'Digital Science Dimensions' 
	(<https://www.dimensions.ai/>), 'OpenAlex' (<https://openalex.org/>), 'Cochrane Library' (<https://www.cochranelibrary.com/>),  'Lens' (<https://lens.org>), 
	and 'PubMed' (<https://pubmed.ncbi.nlm.nih.gov/>) databases, performing bibliometric analysis 
    and building networks for co-citation, coupling, scientific collaboration and co-word analysis.
License: GPL-3
URL: https://www.bibliometrix.org,
        https://github.com/massimoaria/bibliometrix,
        https://www.k-synth.com
BugReports: https://github.com/massimoaria/bibliometrix/issues
LazyData: true
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: stats, grDevices, bibliometrixData, contentanalysis (>=
        1.0.0), dimensionsR, dplyr, ca, forcats, ggplot2, ggrepel,
        igraph, Matrix, methods, plotly, openalexR, openxlsx, pubmedR,
        purrr, readr, readxl, rscopus, shiny, shinycssloaders (>=
        1.1.0), SnowballC, stringdist, stringi, stringr, tibble, tidyr,
        tidytext, visNetwork
Suggests: knitr, ragg, rmarkdown, testthat (>= 3.0.0), wordcloud2
RoxygenNote: 7.3.3
NeedsCompilation: no
Config/testthat/edition: 3
Packaged: 2026-04-08 10:47:11 UTC; massimoaria
Author: Massimo Aria [cre, aut, cph] (ORCID:
    <https://orcid.org/0000-0002-8517-9411>),
  Corrado Cuccurullo [aut] (ORCID:
    <https://orcid.org/0000-0002-7401-8575>)
Maintainer: Massimo Aria <aria@unina.it>
Repository: CRAN
Date/Publication: 2026-04-09 05:10:50 UTC
Built: R 4.5.2; ; 2026-04-09 07:32:36 UTC; unix
