#' List of English stopwords.
#'
#' A character vector containing a complete list of English stopwords\cr
#' Data are used by  \code{\link{biblioAnalysis}} function
#' to extract Country Field of Cited References and Authors.
#'
#' @format A character vector with 665 rows.\cr
#'
#' @name stopwords
NULL
