#' Custom Theme variables for Biblioshiny.
#'
#' List containing a set of custom theme variables for Biblioshiny.
#'
#' @format A list with 3 elements:
#'  \describe{
#'     \item{name}{object name}
#'     \item{attribs}{attributes}
#'     \item{children}{CSS style}
#'     }
#'
#' @name customTheme
NULL
