#' Index of ltwa.
#'
#' Data frame containing a normalized index of words used in journal names and their ISO4 abbreviations.
#'
#' @format A data frame with 56463 rows and 3 variables:
#'  \describe{
#'     \item{WORD}{word from journal names}
#'     \item{ABBREVIATION}{ISO4 abbreviation}
#'     \item{LANGUAGES}{Language of the journal name}
#'     }
#'
#' @name ltwa
NULL
