% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logo.R
\name{logo}
\alias{logo}
\title{Bibliometrix logo.}
\format{
A matrix with 927 rows and 800 columns.
}
\description{
The matrix contains the rgb format of the bibliometrix official logo.\cr
}
