% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{prepare_ltwa_lookup}
\alias{prepare_ltwa_lookup}
\title{Prepare LTWA database for efficient lookup}
\usage{
prepare_ltwa_lookup(ltwa_db)
}
\arguments{
\item{ltwa_db}{LTWA database data frame}
}
\value{
List with singles, prefix, and phrase lookup tables
}
\description{
Pre-processes LTWA database into optimized lookup tables
}
\keyword{internal}
