% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{remove_diacritics}
\alias{remove_diacritics}
\title{Remove diacritics from string with robust fallback}
\usage{
remove_diacritics(x)
}
\description{
Remove diacritics from string with robust fallback
}
\keyword{internal}
