% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotThematicEvolution.R
\name{plotThematicEvolution}
\alias{plotThematicEvolution}
\title{Plot a Thematic Evolution Analysis}
\usage{
plotThematicEvolution(Nodes, Edges, measure = "lineage_strength", min.flow = 0)
}
\arguments{
\item{Nodes}{is a list of nodes obtained by \code{\link{thematicEvolution}} function.}

\item{Edges}{is a list of edges obtained by \code{\link{thematicEvolution}} function.}

\item{measure}{is a character. It is deprecated and has no effect. The weight of the flows is always calculated using lineage_strength.}

\item{min.flow}{is numerical. It indicates the minimum value of measure to plot a flow.}
}
\value{
a sankeyPlot
}
\description{
It plot a Thematic Evolution Analysis performed using the \code{\link{thematicEvolution}} function.
}
\examples{

\dontrun{
data(management, package = "bibliometrixData")
years=c(2004,2008,2015)

nexus <- thematicEvolution(management,field="DE",years=years,n=100,minFreq=2)

plotThematicEvolution(nexus$Nodes,nexus$Edges)
}

}
\seealso{
\code{\link{thematicMap}} function to create a thematic map based on co-word network analysis and clustering.

\code{\link{thematicEvolution}} function to perform a thematic evolution analysis.

\code{\link{networkPlot}} to plot a bibliographic network.
}
