% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/apply_citation_matching.R
\name{normalize_journal_to_iso4}
\alias{normalize_journal_to_iso4}
\title{Normalize journal names to ISO4 abbreviated form}
\usage{
normalize_journal_to_iso4(journal_name, ltwa_db)
}
\arguments{
\item{journal_name}{Character string with journal name}

\item{ltwa_db}{Data frame with LTWA database}
}
\value{
Normalized journal name in ISO4 abbreviated form
}
\description{
Converts all journal names to their ISO4 abbreviated form using LTWA.
Only uses English language entries from LTWA to avoid foreign word matches.
}
\keyword{internal}
